@@ -10,6 +10,7 @@
 #include "config.h"
 
 #include "snapd-client.h"
+#include "snapd-error.h"
 
 typedef struct {
     GMainContext *context;
@@ -74,7 +75,7 @@ snapd_client_connect_sync (SnapdClient *client,
 /**
  * snapd_client_login_sync:
  * @client: a #SnapdClient.
- * @username: usename to log in with.
+ * @email: email address to log in with.
  * @password: password to log in with.
  * @otp: (allow-none): response to one-time password challenge.
  * @cancellable: (allow-none): a #GCancellable or %NULL.
@@ -86,23 +87,152 @@ snapd_client_connect_sync (SnapdClient *client,
  * Returns: (transfer full): a #SnapdAuthData or %NULL on error.
  *
  * Since: 1.0
+ * Deprecated: 1.26: Use snapd_client_login2_sync()
  */
 SnapdAuthData *
 snapd_client_login_sync (SnapdClient *client,
-                         const gchar *username, const gchar *password, const gchar *otp,
+                         const gchar *email, const gchar *password, const gchar *otp,
                          GCancellable *cancellable, GError **error)
 {
     g_auto(SyncData) data = { 0 };
 
     g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
-    g_return_val_if_fail (username != NULL, NULL);
+    g_return_val_if_fail (email != NULL, NULL);
     g_return_val_if_fail (password != NULL, NULL);
 
     start_sync (&data);
-    snapd_client_login_async (client, username, password, otp, cancellable, sync_cb, &data);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    snapd_client_login_async (client, email, password, otp, cancellable, sync_cb, &data);
+G_GNUC_END_IGNORE_DEPRECATIONS
     end_sync (&data);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     return snapd_client_login_finish (client, data.result, error);
+G_GNUC_END_IGNORE_DEPRECATIONS
+}
+
+/**
+ * snapd_client_login2_sync:
+ * @client: a #SnapdClient.
+ * @email: email address to log in with.
+ * @password: password to log in with.
+ * @otp: (allow-none): response to one-time password challenge.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL to ignore.
+ *
+ * Log in to snapd and get authorization to install/remove snaps.
+ * This call requires root access; use snapd_login_sync() if you are non-root.
+ *
+ * Returns: (transfer full): a #SnapdUserInformation or %NULL on error.
+ *
+ * Since: 1.26
+ */
+SnapdUserInformation *
+snapd_client_login2_sync (SnapdClient *client,
+                         const gchar *email, const gchar *password, const gchar *otp,
+                         GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
+    g_return_val_if_fail (email != NULL, NULL);
+    g_return_val_if_fail (password != NULL, NULL);
+
+    start_sync (&data);
+    snapd_client_login2_async (client, email, password, otp, cancellable, sync_cb, &data);
+    end_sync (&data);
+
+    return snapd_client_login2_finish (client, data.result, error);
+}
+
+/**
+ * snapd_client_get_changes_sync:
+ * @client: a #SnapdClient.
+ * @filter: changes to filter on.
+ * @snap_name: (allow-none): name of snap to filter on or %NULL for changes for any snap.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL to ignore.
+ *
+ * Get changes that have occurred / are occurring on the snap daemon.
+ *
+ * Returns: (transfer container) (element-type SnapdChange): an array of #SnapdChange or %NULL on error.
+ *
+ * Since: 1.29
+ */
+GPtrArray *
+snapd_client_get_changes_sync (SnapdClient *client,
+                               SnapdChangeFilter filter, const gchar *snap_name,
+                               GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
+
+    start_sync (&data);
+    snapd_client_get_changes_async (client, filter, snap_name, cancellable, sync_cb, &data);
+    end_sync (&data);
+
+    return snapd_client_get_changes_finish (client, data.result, error);
+}
+
+/**
+ * snapd_client_get_change_sync:
+ * @client: a #SnapdClient.
+ * @id: a change ID to get information on.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL to ignore.
+ *
+ * Get information on a change.
+ *
+ * Returns: (transfer full): a #SnapdChange or %NULL on error.
+ *
+ * Since: 1.29
+ */
+SnapdChange *
+snapd_client_get_change_sync (SnapdClient *client,
+                              const gchar *id,
+                              GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
+    g_return_val_if_fail (id != NULL, NULL);
+
+    start_sync (&data);
+    snapd_client_get_change_async (client, id, cancellable, sync_cb, &data);
+    end_sync (&data);
+
+    return snapd_client_get_change_finish (client, data.result, error);
+}
+
+/**
+ * snapd_client_abort_change_sync:
+ * @client: a #SnapdClient.
+ * @id: a change ID to abort.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL to ignore.
+ *
+ * Get information on a change.
+ *
+ * Returns: (transfer full): a #SnapdChange or %NULL on error.
+ *
+ * Since: 1.30
+ */
+SnapdChange *
+snapd_client_abort_change_sync (SnapdClient *client,
+                                const gchar *id,
+                                GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
+    g_return_val_if_fail (id != NULL, NULL);
+
+    start_sync (&data);
+    snapd_client_abort_change_async (client, id, cancellable, sync_cb, &data);
+    end_sync (&data);
+
+    return snapd_client_abort_change_finish (client, data.result, error);
 }
 
 /**
@@ -161,6 +291,34 @@ snapd_client_list_one_sync (SnapdClient *client,
     return snapd_client_list_one_finish (client, data.result, error);
 }
 
+/**
+ * snapd_client_get_apps_sync:
+ * @client: a #SnapdClient.
+ * @flags: a set of #SnapdGetAppsFlags to control what results are returned.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL to ignore.
+ *
+ * Get information on installed apps.
+ *
+ * Returns: (transfer container) (element-type SnapdApp): an array of #SnapdApp or %NULL on error.
+ *
+ * Since: 1.25
+ */
+GPtrArray *
+snapd_client_get_apps_sync (SnapdClient *client,
+                            SnapdGetAppsFlags flags,
+                            GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
+
+    start_sync (&data);
+    snapd_client_get_apps_async (client, flags, cancellable, sync_cb, &data);
+    end_sync (&data);
+    return snapd_client_get_apps_finish (client, data.result, error);
+}
+
 /**
  * snapd_client_get_icon_sync:
  * @client: a #SnapdClient.
@@ -755,6 +913,39 @@ snapd_client_disable_sync (SnapdClient *client,
     return snapd_client_disable_finish (client, data.result, error);
 }
 
+/**
+ * snapd_client_switch_sync:
+ * @client: a #SnapdClient.
+ * @name: name of snap to switch channel.
+ * @channel: channel to track.
+ * @progress_callback: (allow-none) (scope call): function to callback with progress.
+ * @progress_callback_data: (closure): user data to pass to @progress_callback.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL to ignore.
+ *
+ * Set the tracking channel on an installed snap.
+ *
+ * Returns: %TRUE on success or %FALSE on error.
+ *
+ * Since: 1.26
+ */
+gboolean
+snapd_client_switch_sync (SnapdClient *client,
+                          const gchar *name, const gchar *channel,
+                          SnapdProgressCallback progress_callback, gpointer progress_callback_data,
+                          GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
+    g_return_val_if_fail (name != NULL, FALSE);
+
+    start_sync (&data);
+    snapd_client_switch_async (client, name, channel, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
+    end_sync (&data);
+    return snapd_client_switch_finish (client, data.result, error);
+}
+
 /**
  * snapd_client_check_buy_sync:
  * @client: a #SnapdClient.
@@ -874,6 +1065,33 @@ snapd_client_create_users_sync (SnapdClient *client,
     return snapd_client_create_users_finish (client, data.result, error);
 }
 
+/**
+ * snapd_client_get_users_sync:
+ * @client: a #SnapdClient.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL
+ *     to ignore.
+ *
+ * Get user accounts that are valid for this device.
+ *
+ * Returns: (transfer container) (element-type SnapdUserInformation): an array of #SnapdUserInformation or %NULL on error.
+ *
+ * Since: 1.26
+ */
+GPtrArray *
+snapd_client_get_users_sync (SnapdClient *client,
+                             GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), NULL);
+
+    start_sync (&data);
+    snapd_client_get_users_async (client, cancellable, sync_cb, &data);
+    end_sync (&data);
+    return snapd_client_get_users_finish (client, data.result, error);
+}
+
 /**
  * snapd_client_get_sections_sync:
  * @client: a #SnapdClient.
@@ -928,6 +1146,104 @@ snapd_client_get_aliases_sync (SnapdClient *client,
     return snapd_client_get_aliases_finish (client, data.result, error);
 }
 
+/**
+ * snapd_client_alias_sync:
+ * @client: a #SnapdClient.
+ * @snap: the name of the snap to modify.
+ * @app: an app in the snap to make the alias to.
+ * @alias: the name of the alias (i.e. the command that will run this app).
+ * @progress_callback: (allow-none) (scope async): function to callback with progress.
+ * @progress_callback_data: (closure): user data to pass to @progress_callback.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL
+ *     to ignore.
+ *
+ * Create an alias to an app.
+ *
+ * Since: 1.25
+ */
+gboolean
+snapd_client_alias_sync (SnapdClient *client,
+                         const gchar *snap, const gchar *app, const gchar *alias,
+                         SnapdProgressCallback progress_callback, gpointer progress_callback_data,
+                         GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
+    g_return_val_if_fail (snap != NULL, FALSE);
+    g_return_val_if_fail (app != NULL, FALSE);
+    g_return_val_if_fail (alias != NULL, FALSE);
+
+    start_sync (&data);
+    snapd_client_alias_async (client, snap, app, alias, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
+    end_sync (&data);
+    return snapd_client_alias_finish (client, data.result, error);
+}
+
+/**
+ * snapd_client_unalias_sync:
+ * @client: a #SnapdClient.
+ * @snap: (allow-none): the name of the snap to modify or %NULL.
+ * @alias: (allow-none): the name of the alias to remove or %NULL to remove all aliases for the given snap.
+ * @progress_callback: (allow-none) (scope async): function to callback with progress.
+ * @progress_callback_data: (closure): user data to pass to @progress_callback.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL
+ *     to ignore.
+ *
+ * Remove an alias from an app.
+ *
+ * Since: 1.25
+ */
+gboolean
+snapd_client_unalias_sync (SnapdClient *client,
+                           const gchar *snap, const gchar *alias,
+                           SnapdProgressCallback progress_callback, gpointer progress_callback_data,
+                           GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
+    g_return_val_if_fail (alias != NULL, FALSE);
+
+    start_sync (&data);
+    snapd_client_unalias_async (client, snap, alias, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
+    end_sync (&data);
+    return snapd_client_unalias_finish (client, data.result, error);
+}
+
+/**
+ * snapd_client_prefer_sync:
+ * @client: a #SnapdClient.
+ * @snap: the name of the snap to modify.
+ * @progress_callback: (allow-none) (scope async): function to callback with progress.
+ * @progress_callback_data: (closure): user data to pass to @progress_callback.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): #GError location to store the error occurring, or %NULL
+ *     to ignore.
+ *
+ * ???
+ *
+ * Since: 1.25
+ */
+gboolean
+snapd_client_prefer_sync (SnapdClient *client,
+                          const gchar *snap,
+                          SnapdProgressCallback progress_callback, gpointer progress_callback_data,
+                          GCancellable *cancellable, GError **error)
+{
+    g_auto(SyncData) data = { 0 };
+
+    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
+    g_return_val_if_fail (snap != NULL, FALSE);
+
+    start_sync (&data);
+    snapd_client_prefer_async (client, snap, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
+    end_sync (&data);
+    return snapd_client_prefer_finish (client, data.result, error);
+}
+
 /**
  * snapd_client_enable_aliases_sync:
  * @client: a #SnapdClient.
@@ -944,6 +1260,7 @@ snapd_client_get_aliases_sync (SnapdClient *client,
  * Returns: %TRUE on success or %FALSE on error.
  *
  * Since: 1.8
+ * Deprecated: 1.25: Use snapd_client_alias_sync()
  */
 gboolean
 snapd_client_enable_aliases_sync (SnapdClient *client,
@@ -951,16 +1268,8 @@ snapd_client_enable_aliases_sync (SnapdClient *client,
                                   SnapdProgressCallback progress_callback, gpointer progress_callback_data,
                                   GCancellable *cancellable, GError **error)
 {
-    g_auto(SyncData) data = { 0 };
-
-    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
-    g_return_val_if_fail (snap != NULL, FALSE);
-    g_return_val_if_fail (aliases != NULL, FALSE);
-
-    start_sync (&data);
-    snapd_client_enable_aliases_async (client, snap, aliases, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
-    end_sync (&data);
-    return snapd_client_enable_aliases_finish (client, data.result, error);
+    g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_FAILED, "snapd_client_enable_aliases_async is deprecated");
+    return FALSE;
 }
 
 /**
@@ -979,6 +1288,7 @@ snapd_client_enable_aliases_sync (SnapdClient *client,
  * Returns: %TRUE on success or %FALSE on error.
  *
  * Since: 1.8
+ * Deprecated: 1.25: Use snapd_client_unalias_sync()
  */
 gboolean
 snapd_client_disable_aliases_sync (SnapdClient *client,
@@ -986,16 +1296,8 @@ snapd_client_disable_aliases_sync (SnapdClient *client,
                                    SnapdProgressCallback progress_callback, gpointer progress_callback_data,
                                    GCancellable *cancellable, GError **error)
 {
-    g_auto(SyncData) data = { 0 };
-
-    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
-    g_return_val_if_fail (snap != NULL, FALSE);
-    g_return_val_if_fail (aliases != NULL, FALSE);
-
-    start_sync (&data);
-    snapd_client_disable_aliases_async (client, snap, aliases, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
-    end_sync (&data);
-    return snapd_client_disable_aliases_finish (client, data.result, error);
+    g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_FAILED, "snapd_client_disable_aliases_async is deprecated");
+    return FALSE;
 }
 
 /**
@@ -1014,6 +1316,7 @@ snapd_client_disable_aliases_sync (SnapdClient *client,
  * Returns: %TRUE on success or %FALSE on error.
  *
  * Since: 1.8
+ * Deprecated: 1.25: Use snapd_client_disable_aliases_sync()
  */
 gboolean
 snapd_client_reset_aliases_sync (SnapdClient *client,
@@ -1021,16 +1324,8 @@ snapd_client_reset_aliases_sync (SnapdClient *client,
                                  SnapdProgressCallback progress_callback, gpointer progress_callback_data,
                                  GCancellable *cancellable, GError **error)
 {
-    g_auto(SyncData) data = { 0 };
-
-    g_return_val_if_fail (SNAPD_IS_CLIENT (client), FALSE);
-    g_return_val_if_fail (snap != NULL, FALSE);
-    g_return_val_if_fail (aliases != NULL, FALSE);
-
-    start_sync (&data);
-    snapd_client_reset_aliases_async (client, snap, aliases, progress_callback, progress_callback_data, cancellable, sync_cb, &data);
-    end_sync (&data);
-    return snapd_client_reset_aliases_finish (client, data.result, error);
+    g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_FAILED, "snapd_client_reset_aliases_async is deprecated");
+    return FALSE;
 }
 
 /**
