@@ -15,6 +15,26 @@
 #include "snapd-error.h"
 #include "snapd-app.h"
 #include "snapd-screenshot.h"
+#include "snapd-task.h"
+
+void
+_snapd_json_set_body (SoupMessage *message, JsonBuilder *builder)
+{
+    g_autoptr(JsonNode) json_root = NULL;
+    g_autoptr(JsonGenerator) json_generator = NULL;
+    g_autofree gchar *data = NULL;
+    gsize data_length;
+
+    json_root = json_builder_get_root (builder);
+    json_generator = json_generator_new ();
+    json_generator_set_pretty (json_generator, TRUE);
+    json_generator_set_root (json_generator, json_root);
+    data = json_generator_to_data (json_generator, &data_length);
+
+    soup_message_headers_set_content_type (message->request_headers, "application/json", NULL);
+    soup_message_body_append_take (message->request_body, g_steal_pointer (&data), data_length);
+    soup_message_headers_set_content_length (message->request_headers, message->request_body->length);
+}
 
 gboolean
 _snapd_json_get_bool (JsonObject *object, const gchar *name, gboolean default_value)
@@ -124,8 +144,8 @@ is_timezone_prefix (gchar c)
     return c == '+' || c == '-' || c == 'Z';
 }
 
-GDateTime *
-_snapd_json_get_date_time (JsonObject *object, const gchar *name)
+static GDateTime *
+parse_date_time (JsonObject *object, const gchar *name)
 {
     const gchar *value;
     g_auto(GStrv) tokens = NULL;
@@ -246,16 +266,39 @@ parse_error_response (JsonObject *root, GError **error)
                              SNAPD_ERROR,
                              SNAPD_ERROR_NEEDS_CLASSIC_SYSTEM,
                              message);
-    else if (status_code == SOUP_STATUS_BAD_REQUEST)
-        g_set_error_literal (error,
-                             SNAPD_ERROR,
-                             SNAPD_ERROR_BAD_REQUEST,
-                             message);
-    else
-        g_set_error_literal (error,
-                             SNAPD_ERROR,
-                             SNAPD_ERROR_FAILED,
-                             message);
+    else {
+        switch (status_code) {
+        case SOUP_STATUS_BAD_REQUEST:
+            g_set_error_literal (error,
+                                 SNAPD_ERROR,
+                                 SNAPD_ERROR_BAD_REQUEST,
+                                 message);
+            break;
+        case SOUP_STATUS_UNAUTHORIZED:
+            g_set_error_literal (error,
+                                 SNAPD_ERROR,
+                                 SNAPD_ERROR_AUTH_DATA_REQUIRED,
+                                 message);
+            break;
+        case SOUP_STATUS_FORBIDDEN:
+            g_set_error_literal (error,
+                                 SNAPD_ERROR,
+                                 SNAPD_ERROR_PERMISSION_DENIED,
+                                 message);
+            break;
+        /* Other response codes currently produced by snapd:
+        case SOUP_STATUS_NOT_FOUND:
+        case SOUP_STATUS_METHOD_NOT_ALLOWED:
+        case SOUP_STATUS_NOT_IMPLEMENTED:
+        case SOUP_STATUS_CONFLICT:
+         */
+        default:
+            g_set_error_literal (error,
+                                 SNAPD_ERROR,
+                                 SNAPD_ERROR_FAILED,
+                                 message);
+        }
+    }
 }
 
 JsonObject *
@@ -387,6 +430,66 @@ _snapd_json_get_async_result (JsonObject *response, GError **error)
     return g_strdup (json_node_get_string (change_node));
 }
 
+SnapdChange *
+_snapd_json_parse_change (JsonObject *object, GError **error)
+{
+    g_autoptr(JsonArray) array = NULL;
+    guint i;
+    g_autoptr(GPtrArray) tasks = NULL;
+    g_autoptr(GDateTime) main_spawn_time = NULL;
+    g_autoptr(GDateTime) main_ready_time = NULL;
+
+    array = _snapd_json_get_array (object, "tasks");
+    tasks = g_ptr_array_new_with_free_func (g_object_unref);
+    for (i = 0; i < json_array_get_length (array); i++) {
+        JsonNode *node = json_array_get_element (array, i);
+        JsonObject *object, *progress;
+        g_autoptr(GDateTime) spawn_time = NULL;
+        g_autoptr(GDateTime) ready_time = NULL;
+        g_autoptr(SnapdTask) t = NULL;
+
+        if (json_node_get_value_type (node) != JSON_TYPE_OBJECT) {
+            g_set_error (error,
+                         SNAPD_ERROR,
+                         SNAPD_ERROR_READ_FAILED,
+                         "Unexpected task type");
+            return NULL;
+        }
+        object = json_node_get_object (node);
+        progress = _snapd_json_get_object (object, "progress");
+        spawn_time = parse_date_time (object, "spawn-time");
+        ready_time = parse_date_time (object, "ready-time");
+
+        t = g_object_new (SNAPD_TYPE_TASK,
+                          "id", _snapd_json_get_string (object, "id", NULL),
+                          "kind", _snapd_json_get_string (object, "kind", NULL),
+                          "summary", _snapd_json_get_string (object, "summary", NULL),
+                          "status", _snapd_json_get_string (object, "status", NULL),
+                          "progress-label", progress != NULL ? _snapd_json_get_string (progress, "label", NULL) : NULL,
+                          "progress-done", progress != NULL ? _snapd_json_get_int (progress, "done", 0) : 0,
+                          "progress-total", progress != NULL ? _snapd_json_get_int (progress, "total", 0) : 0,
+                          "spawn-time", spawn_time,
+                          "ready-time", ready_time,
+                          NULL);
+        g_ptr_array_add (tasks, g_steal_pointer (&t));
+    }
+
+    main_spawn_time = parse_date_time (object, "spawn-time");
+    main_ready_time = parse_date_time (object, "ready-time");
+
+    return g_object_new (SNAPD_TYPE_CHANGE,
+                         "id", _snapd_json_get_string (object, "id", NULL),
+                         "kind", _snapd_json_get_string (object, "kind", NULL),
+                         "summary", _snapd_json_get_string (object, "summary", NULL),
+                         "status", _snapd_json_get_string (object, "status", NULL),
+                         "tasks", tasks,
+                         "ready", _snapd_json_get_bool (object, "ready", FALSE),
+                         "spawn-time", main_spawn_time,
+                         "ready-time", main_ready_time,
+                         "error", _snapd_json_get_string (object, "err", NULL),
+                         NULL);
+}
+
 static SnapdConfinement
 parse_confinement (const gchar *value)
 {
@@ -400,9 +503,63 @@ parse_confinement (const gchar *value)
         return SNAPD_CONFINEMENT_UNKNOWN;
 }
 
+static GPtrArray *
+_snapd_json_parse_apps (const gchar *snap_name, JsonArray *apps, GError **error)
+{
+    g_autoptr(GPtrArray) apps_array = NULL;
+    guint i;
+
+    apps_array = g_ptr_array_new_with_free_func (g_object_unref);
+    for (i = 0; i < json_array_get_length (apps); i++) {
+        JsonNode *node = json_array_get_element (apps, i);
+        JsonObject *a;
+        const gchar *daemon;
+        g_autoptr(SnapdApp) app = NULL;
+        SnapdDaemonType daemon_type = SNAPD_DAEMON_TYPE_NONE;
+        const gchar *app_snap_name;
+
+        if (json_node_get_value_type (node) != JSON_TYPE_OBJECT) {
+            g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_READ_FAILED, "Unexpected app type");
+            return NULL;
+        }
+
+        a = json_node_get_object (node);
+
+        daemon = _snapd_json_get_string (a, "daemon", NULL);
+        if (daemon == NULL)
+            daemon_type = SNAPD_DAEMON_TYPE_NONE;
+        else if (strcmp (daemon, "simple") == 0)
+            daemon_type = SNAPD_DAEMON_TYPE_SIMPLE;
+        else if (strcmp (daemon, "forking") == 0)
+            daemon_type = SNAPD_DAEMON_TYPE_FORKING;
+        else if (strcmp (daemon, "oneshot") == 0)
+            daemon_type = SNAPD_DAEMON_TYPE_ONESHOT;
+        else if (strcmp (daemon, "dbus") == 0)
+            daemon_type = SNAPD_DAEMON_TYPE_DBUS;
+        else if (strcmp (daemon, "notify") == 0)
+            daemon_type = SNAPD_DAEMON_TYPE_NOTIFY;
+        else
+            daemon_type = SNAPD_DAEMON_TYPE_UNKNOWN;
+
+        app_snap_name = _snapd_json_get_string (a, "snap", NULL);
+        app = g_object_new (SNAPD_TYPE_APP,
+                            "name", _snapd_json_get_string (a, "name", NULL),
+                            "active", _snapd_json_get_bool (a, "active", FALSE),
+                            "daemon-type", daemon_type,
+                            "desktop-file", _snapd_json_get_string (a, "desktop-file", NULL),
+                            "enabled", _snapd_json_get_bool (a, "enabled", FALSE),
+                            "snap", app_snap_name != NULL ? app_snap_name : snap_name,
+                            NULL);
+        g_ptr_array_add (apps_array, g_steal_pointer (&app));
+    }
+
+    return g_steal_pointer (&apps_array);
+}
+
 SnapdSnap *
 _snapd_json_parse_snap (JsonObject *object, GError **error)
 {
+    const gchar *name;
     SnapdConfinement confinement;
     const gchar *snap_type_string;
     SnapdSnapType snap_type = SNAPD_SNAP_TYPE_UNKNOWN;
@@ -421,6 +578,8 @@ _snapd_json_parse_snap (JsonObject *object, GError **error)
     g_autoptr(GPtrArray) track_array = NULL;
     guint i;
 
+    name = _snapd_json_get_string (object, "name", NULL);
+
     confinement = parse_confinement (_snapd_json_get_string (object, "confinement", ""));
 
     snap_type_string = _snapd_json_get_string (object, "type", "");
@@ -444,62 +603,9 @@ _snapd_json_parse_snap (JsonObject *object, GError **error)
         snap_status = SNAPD_SNAP_STATUS_ACTIVE;
 
     apps = _snapd_json_get_array (object, "apps");
-    apps_array = g_ptr_array_new_with_free_func (g_object_unref);
-    for (i = 0; i < json_array_get_length (apps); i++) {
-        JsonNode *node = json_array_get_element (apps, i);
-        JsonObject *a;
-        g_autoptr(JsonArray) aliases = NULL;
-        g_autoptr(GPtrArray) aliases_array = NULL;
-        int j;
-        const gchar *daemon;
-        g_autoptr(SnapdApp) app = NULL;
-        SnapdDaemonType daemon_type = SNAPD_DAEMON_TYPE_NONE;
-
-        if (json_node_get_value_type (node) != JSON_TYPE_OBJECT) {
-            g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_READ_FAILED, "Unexpected app type");
-            return NULL;
-        }
-
-        a = json_node_get_object (node);
-
-        aliases = _snapd_json_get_array (a, "aliases");
-        aliases_array = g_ptr_array_new ();
-        for (j = 0; j < json_array_get_length (aliases); j++) {
-            JsonNode *node = json_array_get_element (aliases, j);
-
-            if (json_node_get_value_type (node) != G_TYPE_STRING) {
-                g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_READ_FAILED, "Unexpected alias type");
-                return NULL;
-            }
-
-            g_ptr_array_add (aliases_array, (gpointer) json_node_get_string (node));
-        }
-        g_ptr_array_add (aliases_array, NULL);
-
-        daemon = _snapd_json_get_string (a, "daemon", NULL);
-        if (daemon == NULL)
-            daemon_type = SNAPD_DAEMON_TYPE_NONE;
-        else if (strcmp (daemon, "simple") == 0)
-            daemon_type = SNAPD_DAEMON_TYPE_SIMPLE;
-        else if (strcmp (daemon, "forking") == 0)
-            daemon_type = SNAPD_DAEMON_TYPE_FORKING;
-        else if (strcmp (daemon, "oneshot") == 0)
-            daemon_type = SNAPD_DAEMON_TYPE_ONESHOT;
-        else if (strcmp (daemon, "dbus") == 0)
-            daemon_type = SNAPD_DAEMON_TYPE_DBUS;
-        else if (strcmp (daemon, "notify") == 0)
-            daemon_type = SNAPD_DAEMON_TYPE_NOTIFY;
-        else
-            daemon_type = SNAPD_DAEMON_TYPE_UNKNOWN;
-
-        app = g_object_new (SNAPD_TYPE_APP,
-                            "name", _snapd_json_get_string (a, "name", NULL),
-                            "aliases", (gchar **) aliases_array->pdata,
-                            "daemon-type", daemon_type,
-                            "desktop-file", _snapd_json_get_string (a, "desktop-file", NULL),
-                            NULL);
-        g_ptr_array_add (apps_array, g_steal_pointer (&app));
-    }
+    apps_array = _snapd_json_parse_apps (name, apps, error);
+    if (apps_array == NULL)
+        return NULL;
 
     channels = _snapd_json_get_object (object, "channels");
     channels_array = g_ptr_array_new_with_free_func (g_object_unref);
@@ -534,7 +640,7 @@ _snapd_json_parse_snap (JsonObject *object, GError **error)
         }
     }
 
-    install_date = _snapd_json_get_date_time (object, "install-date");
+    install_date = parse_date_time (object, "install-date");
 
     prices = _snapd_json_get_object (object, "prices");
     prices_array = g_ptr_array_new_with_free_func (g_object_unref);
@@ -581,7 +687,11 @@ _snapd_json_parse_snap (JsonObject *object, GError **error)
         g_ptr_array_add (screenshots_array, g_steal_pointer (&screenshot));
     }
 
-    tracks = _snapd_json_get_array (object, "tracks");
+    /* The tracks field was originally incorrectly named, fixed in snapd 61ad9ed (2.29.5) */
+    if (json_object_has_member (object, "Tracks"))
+        tracks = _snapd_json_get_array (object, "Tracks");
+    else
+        tracks = _snapd_json_get_array (object, "tracks");
     track_array = g_ptr_array_new ();
     for (i = 0; i < json_array_get_length (tracks); i++) {
         JsonNode *node = json_array_get_element (tracks, i);
@@ -597,6 +707,7 @@ _snapd_json_parse_snap (JsonObject *object, GError **error)
 
     return g_object_new (SNAPD_TYPE_SNAP,
                          "apps", apps_array,
+                         "broken", _snapd_json_get_string (object, "broken", NULL),
                          "channel", _snapd_json_get_string (object, "channel", NULL),
                          "channels", channels_array,
                          "confinement", confinement,
@@ -611,7 +722,7 @@ _snapd_json_parse_snap (JsonObject *object, GError **error)
                          "installed-size", _snapd_json_get_int (object, "installed-size", 0),
                          "jailmode", _snapd_json_get_bool (object, "jailmode", FALSE),
                          "license", _snapd_json_get_string (object, "license", NULL),
-                         "name", _snapd_json_get_string (object, "name", NULL),
+                         "name", name,
                          "prices", prices_array,
                          "private", _snapd_json_get_bool (object, "private", FALSE),
                          "revision", _snapd_json_get_string (object, "revision", NULL),
@@ -652,11 +763,18 @@ _snapd_json_parse_snap_array (JsonArray *array, GError **error)
     return g_steal_pointer (&snaps);
 }
 
+GPtrArray *
+_snapd_json_parse_app_array (JsonArray *array, GError **error)
+{
+    return _snapd_json_parse_apps (NULL, array, error);
+}
+
 SnapdUserInformation *
 _snapd_json_parse_user_information (JsonObject *object, GError **error)
 {
     g_autoptr(JsonArray) ssh_keys = NULL;
     g_autoptr(GPtrArray) ssh_key_array = NULL;
+    g_autoptr(SnapdAuthData) auth_data = NULL;
     guint i;
 
     ssh_keys = _snapd_json_get_array (object, "ssh-keys");
@@ -671,10 +789,37 @@ _snapd_json_parse_user_information (JsonObject *object, GError **error)
 
         g_ptr_array_add (ssh_key_array, (gpointer) json_node_get_string (node));
     }
+
     g_ptr_array_add (ssh_key_array, NULL);
 
+    if (json_object_has_member (object, "macaroon")) {
+        g_autoptr(JsonArray) discharges = NULL;
+        g_autoptr(GPtrArray) discharge_array = NULL;
+
+        discharges = _snapd_json_get_array (object, "discharges");
+        discharge_array = g_ptr_array_new ();
+        for (i = 0; i < json_array_get_length (discharges); i++) {
+            JsonNode *node = json_array_get_element (discharges, i);
+
+            if (json_node_get_value_type (node) != G_TYPE_STRING) {
+                g_set_error (error,
+                             SNAPD_ERROR,
+                             SNAPD_ERROR_READ_FAILED,
+                             "Unexpected discharge type");
+                return FALSE;
+            }
+
+            g_ptr_array_add (discharge_array, (gpointer) json_node_get_string (node));
+        }
+        g_ptr_array_add (discharge_array, NULL);
+        auth_data = snapd_auth_data_new (_snapd_json_get_string (object, "macaroon", NULL), (gchar **) discharge_array->pdata);
+    }
+
     return g_object_new (SNAPD_TYPE_USER_INFORMATION,
+                         "id", _snapd_json_get_int (object, "id", -1),
                          "username", _snapd_json_get_string (object, "username", NULL),
+                         "email", _snapd_json_get_string (object, "email", NULL),
                          "ssh-keys", (gchar **) ssh_key_array->pdata,
+                         "auth-data", auth_data,
                          NULL);
 }
