@@ -35,6 +35,7 @@ struct _SnapdSnap
     GObject parent_instance;
 
     GPtrArray *apps;
+    gchar *broken;
     gchar *channel;
     GPtrArray *channels;
     SnapdConfinement confinement;
@@ -94,6 +95,7 @@ enum
     PROP_LICENSE,
     PROP_CHANNELS,
     PROP_TRACKS,
+    PROP_BROKEN,
     PROP_LAST
 };
 
@@ -116,6 +118,23 @@ snapd_snap_get_apps (SnapdSnap *snap)
     return snap->apps;
 }
 
+/**
+ * snapd_snap_get_broken:
+ * @snap: a #SnapdSnap.
+ *
+ * Get the reason this snap is broken.
+ *
+ * Returns: an error string or %NULL if not broken.
+ *
+ * Since: 1.25
+ */
+const gchar *
+snapd_snap_get_broken (SnapdSnap *snap)
+{
+    g_return_val_if_fail (SNAPD_IS_SNAP (snap), NULL);
+    return snap->broken;
+}
+
 /**
  * snapd_snap_get_channel:
  * @snap: a #SnapdSnap.
@@ -342,7 +361,7 @@ snapd_snap_get_developer (SnapdSnap *snap)
  * snapd_snap_get_devmode:
  * @snap: a #SnapdSnap.
  *
- * Get if this snap is running in developmer mode.
+ * Get if this snap is running in developer mode.
  *
  * Returns: %TRUE if this snap is running in devmode.
  *
@@ -653,7 +672,7 @@ snapd_snap_get_tracking_channel (SnapdSnap *snap)
  *
  * Get the tracks that are available.
  *
- * Returns: (transfer none) (array zero-terminated=1): an orderered array of track names.
+ * Returns: (transfer none) (array zero-terminated=1): an ordered array of track names.
  *
  * Since: 1.22
  */
@@ -711,6 +730,10 @@ snapd_snap_set_property (GObject *object, guint prop_id, const GValue *value, GP
         if (g_value_get_boxed (value) != NULL)
             snap->apps = g_ptr_array_ref (g_value_get_boxed (value));
         break;
+    case PROP_BROKEN:
+        g_free (snap->broken);
+        snap->broken = g_strdup (g_value_get_string (value));
+        break;
     case PROP_CHANNEL:
         g_free (snap->channel);
         snap->channel = g_strdup (g_value_get_string (value));
@@ -829,6 +852,9 @@ snapd_snap_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
     case PROP_APPS:
         g_value_set_boxed (value, snap->apps);
         break;
+    case PROP_BROKEN:
+        g_value_set_string (value, snap->broken);
+        break;
     case PROP_CHANNEL:
         g_value_set_string (value, snap->channel);
         break;
@@ -922,6 +948,7 @@ snapd_snap_finalize (GObject *object)
     SnapdSnap *snap = SNAPD_SNAP (object);
 
     g_clear_pointer (&snap->apps, g_ptr_array_unref);
+    g_clear_pointer (&snap->broken, g_free);
     g_clear_pointer (&snap->channel, g_free);
     g_clear_pointer (&snap->channels, g_ptr_array_unref);
     g_clear_pointer (&snap->contact, g_free);
@@ -940,6 +967,8 @@ snapd_snap_finalize (GObject *object)
     g_clear_pointer (&snap->tracking_channel, g_free);
     g_clear_pointer (&snap->tracks, g_strfreev);
     g_clear_pointer (&snap->version, g_free);
+
+    G_OBJECT_CLASS (snapd_snap_parent_class)->finalize (object);
 }
 
 static void
@@ -958,6 +987,13 @@ snapd_snap_class_init (SnapdSnapClass *klass)
                                                          "Apps this snap contains",
                                                          G_TYPE_PTR_ARRAY,
                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+    g_object_class_install_property (gobject_class,
+                                     PROP_BROKEN,
+                                     g_param_spec_string ("broken",
+                                                          "broken",
+                                                          "Error string if snap is broken",
+                                                          NULL,
+                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (gobject_class,
                                      PROP_CHANNEL,
                                      g_param_spec_string ("channel",
@@ -1125,7 +1161,7 @@ snapd_snap_class_init (SnapdSnapClass *klass)
                                                          "tracks",
                                                          "Track names",
                                                          G_TYPE_STRV,
-                                                         G_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (gobject_class,
                                      PROP_TRYMODE,
                                      g_param_spec_boolean ("trymode",
