@@ -22,6 +22,7 @@
 #include <Snapd/Slot>
 #include <Snapd/Snap>
 #include <Snapd/SystemInformation>
+#include <Snapd/UserInformation>
 
 class QSnapdConnectRequestPrivate;
 class Q_DECL_EXPORT QSnapdConnectRequest : public QSnapdRequest
@@ -44,13 +45,15 @@ class QSnapdLoginRequestPrivate;
 class Q_DECL_EXPORT QSnapdLoginRequest : public QSnapdRequest
 {
     Q_OBJECT
+    Q_PROPERTY (QSnapdUserInformation* userInformation READ userInformation)
     Q_PROPERTY (QSnapdAuthData* authData READ authData)
 
 public:
-    explicit QSnapdLoginRequest (void *snapd_client, const QString& username, const QString& password, const QString& otp, QObject *parent = 0);
+    explicit QSnapdLoginRequest (void *snapd_client, const QString& email, const QString& password, const QString& otp, QObject *parent = 0);
     ~QSnapdLoginRequest ();
     virtual void runSync ();
     virtual void runAsync ();
+    Q_INVOKABLE QSnapdUserInformation *userInformation ();
     Q_INVOKABLE QSnapdAuthData *authData ();
     void handleResult (void *, void *);
 
@@ -59,6 +62,62 @@ private:
     Q_DECLARE_PRIVATE(QSnapdLoginRequest)
 };
 
+class QSnapdGetChangesRequestPrivate;
+class Q_DECL_EXPORT QSnapdGetChangesRequest : public QSnapdRequest
+{
+    Q_OBJECT
+    Q_PROPERTY(int changeCount READ changeCount)
+
+public:
+    explicit QSnapdGetChangesRequest (int filter, const QString& snapName, void *snapd_client, QObject *parent = 0);
+    ~QSnapdGetChangesRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE int changeCount () const;
+    Q_INVOKABLE QSnapdChange *change (int) const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdGetChangesRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdGetChangesRequest)
+};
+
+class QSnapdGetChangeRequestPrivate;
+class Q_DECL_EXPORT QSnapdGetChangeRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdGetChangeRequest (const QString& id, void *snapd_client, QObject *parent = 0);
+    ~QSnapdGetChangeRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE QSnapdChange *change () const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdGetChangeRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdGetChangeRequest)
+};
+
+class QSnapdAbortChangeRequestPrivate;
+class Q_DECL_EXPORT QSnapdAbortChangeRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdAbortChangeRequest (const QString& id, void *snapd_client, QObject *parent = 0);
+    ~QSnapdAbortChangeRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE QSnapdChange *change () const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdAbortChangeRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdAbortChangeRequest)
+};
+
 class QSnapdGetSystemInformationRequestPrivate;
 class Q_DECL_EXPORT QSnapdGetSystemInformationRequest : public QSnapdRequest
 {
@@ -116,6 +175,26 @@ private:
     Q_DECLARE_PRIVATE(QSnapdListOneRequest)
 };
 
+class QSnapdGetAppsRequestPrivate;
+class Q_DECL_EXPORT QSnapdGetAppsRequest : public QSnapdRequest
+{
+    Q_OBJECT
+    Q_PROPERTY(int appCount READ appCount)
+
+public:
+    explicit QSnapdGetAppsRequest (int flags, void *snapd_client, QObject *parent = 0);
+    ~QSnapdGetAppsRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE int appCount () const;
+    Q_INVOKABLE QSnapdApp *app (int) const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdGetAppsRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdGetAppsRequest)
+};
+
 class QSnapdGetIconRequestPrivate;
 class Q_DECL_EXPORT QSnapdGetIconRequest : public QSnapdRequest
 {
@@ -389,6 +468,23 @@ private:
     Q_DECLARE_PRIVATE(QSnapdDisableRequest)
 };
 
+class QSnapdSwitchChannelRequestPrivate;
+class Q_DECL_EXPORT QSnapdSwitchChannelRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdSwitchChannelRequest (const QString& name, const QString& channel, void *snapd_client, QObject *parent = 0);
+    ~QSnapdSwitchChannelRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    void handleResult (void *, void *);
+
+private:
+    QSnapdSwitchChannelRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdSwitchChannelRequest)
+};
+
 class QSnapdCheckBuyRequestPrivate;
 class Q_DECL_EXPORT QSnapdCheckBuyRequest : public QSnapdRequest
 {
@@ -425,6 +521,64 @@ private:
     Q_DECLARE_PRIVATE(QSnapdBuyRequest)
 };
 
+class QSnapdCreateUserRequestPrivate;
+class Q_DECL_EXPORT QSnapdCreateUserRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdCreateUserRequest (const QString& email, int flags, void *snapd_client, QObject *parent = 0);
+    ~QSnapdCreateUserRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE QSnapdUserInformation *userInformation () const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdCreateUserRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdCreateUserRequest)
+};
+
+class QSnapdCreateUsersRequestPrivate;
+class Q_DECL_EXPORT QSnapdCreateUsersRequest : public QSnapdRequest
+{
+    Q_OBJECT
+    Q_PROPERTY(int userInformationCount READ userInformationCount)
+
+public:
+    explicit QSnapdCreateUsersRequest (void *snapd_client, QObject *parent = 0);
+    ~QSnapdCreateUsersRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE int userInformationCount () const;
+    Q_INVOKABLE QSnapdUserInformation *userInformation (int) const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdCreateUsersRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdCreateUsersRequest)
+};
+
+class QSnapdGetUsersRequestPrivate;
+class Q_DECL_EXPORT QSnapdGetUsersRequest : public QSnapdRequest
+{
+    Q_OBJECT
+    Q_PROPERTY(int userInformationCount READ userInformationCount)
+
+public:
+    explicit QSnapdGetUsersRequest (void *snapd_client, QObject *parent = 0);
+    ~QSnapdGetUsersRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE int userInformationCount () const;
+    Q_INVOKABLE QSnapdUserInformation *userInformation (int) const;
+    void handleResult (void *, void *);
+
+private:
+    QSnapdGetUsersRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdGetUsersRequest)
+};
+
 class QSnapdGetSectionsRequestPrivate;
 class Q_DECL_EXPORT QSnapdGetSectionsRequest : public QSnapdRequest
 {
@@ -463,6 +617,57 @@ private:
     Q_DECLARE_PRIVATE(QSnapdGetAliasesRequest)
 };
 
+class QSnapdAliasRequestPrivate;
+class Q_DECL_EXPORT QSnapdAliasRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdAliasRequest (const QString& snap, const QString& app, const QString& alias, void *snapd_client, QObject *parent = 0);
+    ~QSnapdAliasRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    void handleResult (void *, void *);
+
+private:
+    QSnapdAliasRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdAliasRequest)
+};
+
+class QSnapdUnaliasRequestPrivate;
+class Q_DECL_EXPORT QSnapdUnaliasRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdUnaliasRequest (const QString& snap, const QString& alias, void *snapd_client, QObject *parent = 0);
+    ~QSnapdUnaliasRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    void handleResult (void *, void *);
+
+private:
+    QSnapdUnaliasRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdUnaliasRequest)
+};
+
+class QSnapdPreferRequestPrivate;
+class Q_DECL_EXPORT QSnapdPreferRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdPreferRequest (const QString& snap, void *snapd_client, QObject *parent = 0);
+    ~QSnapdPreferRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    void handleResult (void *, void *);
+
+private:
+    QSnapdPreferRequestPrivate *d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdPreferRequest)
+};
+
 class QSnapdEnableAliasesRequestPrivate;
 class Q_DECL_EXPORT QSnapdEnableAliasesRequest : public QSnapdRequest
 {
@@ -533,8 +738,8 @@ private:
     Q_DECLARE_PRIVATE(QSnapdRunSnapCtlRequest)
 };
 
-Q_INVOKABLE QSnapdLoginRequest *login (const QString& username, const QString& password);
-Q_INVOKABLE QSnapdLoginRequest *login (const QString& username, const QString& password, const QString& otp);
+Q_INVOKABLE QSnapdLoginRequest *login (const QString& email, const QString& password);
+Q_INVOKABLE QSnapdLoginRequest *login (const QString& email, const QString& password, const QString& otp);
 
 class QSnapdClientPrivate;
 class Q_DECL_EXPORT QSnapdClient : public QObject
@@ -547,6 +752,18 @@ class Q_DECL_EXPORT QSnapdClient : public QObject
     Q_FLAGS(FindFlags)
 
 public:
+    enum ChangeFilter
+    {
+        FilterAll,
+        FilterInProgress,
+        FilterReady
+    };
+    Q_ENUM(ChangeFilter)
+    enum GetAppsFlag
+    {
+        SelectServices = 1 << 0
+    };
+    Q_DECLARE_FLAGS(GetAppsFlags, GetAppsFlag);
     enum FindFlag
     {
         None          = 0,
@@ -562,12 +779,18 @@ public:
         Jailmode       = 1 << 3
     };
     Q_DECLARE_FLAGS(InstallFlags, InstallFlag);
+    enum CreateUserFlag
+    {
+        Sudo           = 1 << 0,
+        Known          = 1 << 1
+    };
+    Q_DECLARE_FLAGS(CreateUserFlags, CreateUserFlag);
     explicit QSnapdClient (QObject* parent=0);
     explicit QSnapdClient (int fd, QObject* parent=0);
     virtual ~QSnapdClient ();
     Q_INVOKABLE Q_DECL_DEPRECATED QSnapdConnectRequest *connect ();
-    Q_INVOKABLE QSnapdLoginRequest *login (const QString& username, const QString& password);
-    Q_INVOKABLE QSnapdLoginRequest *login (const QString& username, const QString& password, const QString& otp);
+    Q_INVOKABLE QSnapdLoginRequest *login (const QString& email, const QString& password);
+    Q_INVOKABLE QSnapdLoginRequest *login (const QString& email, const QString& password, const QString& otp);
     Q_INVOKABLE void setSocketPath (const QString &socketPath);
     Q_INVOKABLE QString socketPath () const;
     Q_INVOKABLE void setUserAgent (const QString &userAgent);
@@ -576,9 +799,17 @@ public:
     Q_INVOKABLE bool allowInteraction () const;
     Q_INVOKABLE void setAuthData (QSnapdAuthData *authData);
     Q_INVOKABLE QSnapdAuthData *authData ();
+    Q_INVOKABLE QSnapdGetChangesRequest *getChanges ();
+    Q_INVOKABLE QSnapdGetChangesRequest *getChanges (ChangeFilter filter);
+    Q_INVOKABLE QSnapdGetChangesRequest *getChanges (const QString &snapName);
+    Q_INVOKABLE QSnapdGetChangesRequest *getChanges (ChangeFilter filter, const QString &snapName);
+    Q_INVOKABLE QSnapdGetChangeRequest *getChange (const QString &id);
+    Q_INVOKABLE QSnapdAbortChangeRequest *abortChange (const QString &id);
     Q_INVOKABLE QSnapdGetSystemInformationRequest *getSystemInformation ();
     Q_INVOKABLE QSnapdListRequest *list ();
     Q_INVOKABLE QSnapdListOneRequest *listOne (const QString &name);
+    Q_INVOKABLE QSnapdGetAppsRequest *getApps ();
+    Q_INVOKABLE QSnapdGetAppsRequest *getApps (GetAppsFlags flags);
     Q_INVOKABLE QSnapdGetIconRequest *getIcon (const QString &name);
     Q_INVOKABLE QSnapdGetAssertionsRequest *getAssertions (const QString &type);
     Q_INVOKABLE QSnapdAddAssertionsRequest *addAssertions (const QStringList &assertions);
@@ -604,13 +835,22 @@ public:
     Q_INVOKABLE QSnapdRemoveRequest *remove (const QString &name);
     Q_INVOKABLE QSnapdEnableRequest *enable (const QString &name);
     Q_INVOKABLE QSnapdDisableRequest *disable (const QString &name);
+    Q_INVOKABLE QSnapdSwitchChannelRequest *switchChannel (const QString &name, const QString &channel);
     Q_INVOKABLE QSnapdCheckBuyRequest *checkBuy ();
     Q_INVOKABLE QSnapdBuyRequest *buy (const QString& id, double amount, const QString& currency);
+    Q_INVOKABLE QSnapdCreateUserRequest *createUser (const QString& email);
+    Q_INVOKABLE QSnapdCreateUserRequest *createUser (const QString& email, CreateUserFlags flags);
+    Q_INVOKABLE QSnapdCreateUsersRequest *createUsers ();
+    Q_INVOKABLE QSnapdGetUsersRequest *getUsers ();
     Q_INVOKABLE QSnapdGetSectionsRequest *getSections ();
     Q_INVOKABLE QSnapdGetAliasesRequest *getAliases ();
-    Q_INVOKABLE QSnapdEnableAliasesRequest *enableAliases (const QString snap, const QStringList &aliases);
-    Q_INVOKABLE QSnapdDisableAliasesRequest *disableAliases (const QString snap, const QStringList &aliases);
-    Q_INVOKABLE QSnapdResetAliasesRequest *resetAliases (const QString snap, const QStringList &aliases);
+    Q_INVOKABLE QSnapdAliasRequest *alias (const QString &snap, const QString &app, const QString &alias);
+    Q_INVOKABLE QSnapdUnaliasRequest *unalias (const QString &snap, const QString &alias);
+    Q_INVOKABLE QSnapdUnaliasRequest *unalias (const QString &alias);
+    Q_INVOKABLE QSnapdPreferRequest *prefer (const QString &snap);
+    Q_INVOKABLE Q_DECL_DEPRECATED QSnapdEnableAliasesRequest *enableAliases (const QString snap, const QStringList &aliases);
+    Q_INVOKABLE Q_DECL_DEPRECATED QSnapdDisableAliasesRequest *disableAliases (const QString snap, const QStringList &aliases);
+    Q_INVOKABLE Q_DECL_DEPRECATED QSnapdResetAliasesRequest *resetAliases (const QString snap, const QStringList &aliases);
     Q_INVOKABLE QSnapdRunSnapCtlRequest *runSnapCtl (const QString contextId, const QStringList &args);
 
 private:
@@ -618,7 +858,9 @@ private:
     Q_DECLARE_PRIVATE(QSnapdClient)
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(QSnapdClient::GetAppsFlags)
 Q_DECLARE_OPERATORS_FOR_FLAGS(QSnapdClient::FindFlags)
 Q_DECLARE_OPERATORS_FOR_FLAGS(QSnapdClient::InstallFlags)
+Q_DECLARE_OPERATORS_FOR_FLAGS(QSnapdClient::CreateUserFlags)
 
 #endif
