@@ -64,26 +64,26 @@ QSnapdLoginRequest::~QSnapdLoginRequest ()
     delete d_ptr;
 }
 
-QSnapdLoginRequest *login (const QString& username, const QString& password)
+QSnapdLoginRequest *login (const QString& email, const QString& password)
 {
-    return new QSnapdLoginRequest (NULL, username, password, NULL);
+    return new QSnapdLoginRequest (NULL, email, password, NULL);
 }
 
-QSnapdLoginRequest *login (const QString& username, const QString& password, const QString& otp)
+QSnapdLoginRequest *login (const QString& email, const QString& password, const QString& otp)
 {
-    return new QSnapdLoginRequest (NULL, username, password, otp);
+    return new QSnapdLoginRequest (NULL, email, password, otp);
 }
 
-QSnapdLoginRequest *QSnapdClient::login (const QString& username, const QString& password)
+QSnapdLoginRequest *QSnapdClient::login (const QString& email, const QString& password)
 {
     Q_D(QSnapdClient);
-    return new QSnapdLoginRequest (d->client, username, password, NULL);
+    return new QSnapdLoginRequest (d->client, email, password, NULL);
 }
 
-QSnapdLoginRequest *QSnapdClient::login (const QString& username, const QString& password, const QString& otp)
+QSnapdLoginRequest *QSnapdClient::login (const QString& email, const QString& password, const QString& otp)
 {
     Q_D(QSnapdClient);
-    return new QSnapdLoginRequest (d->client, username, password, otp);
+    return new QSnapdLoginRequest (d->client, email, password, otp);
 }
 
 void QSnapdClient::setSocketPath (const QString &socketPath)
@@ -135,6 +135,57 @@ QSnapdAuthData *QSnapdClient::authData ()
     return new QSnapdAuthData (snapd_client_get_auth_data (d->client));
 }
 
+QSnapdGetChangesRequest::~QSnapdGetChangesRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdGetChangesRequest *QSnapdClient::getChanges ()
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetChangesRequest (FilterAll, NULL, d->client);
+}
+
+QSnapdGetChangesRequest *QSnapdClient::getChanges (ChangeFilter filter)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetChangesRequest (filter, NULL, d->client);
+}
+
+QSnapdGetChangesRequest *QSnapdClient::getChanges (const QString &snapName)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetChangesRequest (FilterAll, snapName, d->client);
+}
+
+QSnapdGetChangesRequest *QSnapdClient::getChanges (ChangeFilter filter, const QString &snapName)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetChangesRequest (filter, snapName, d->client);
+}
+
+QSnapdGetChangeRequest::~QSnapdGetChangeRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdGetChangeRequest *QSnapdClient::getChange (const QString& id)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetChangeRequest (id, d->client);
+}
+
+QSnapdAbortChangeRequest::~QSnapdAbortChangeRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdAbortChangeRequest *QSnapdClient::abortChange (const QString& id)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdAbortChangeRequest (id, d->client);
+}
+
 QSnapdGetSystemInformationRequest::~QSnapdGetSystemInformationRequest ()
 {
     delete d_ptr;
@@ -168,6 +219,23 @@ QSnapdListOneRequest *QSnapdClient::listOne (const QString& name)
     return new QSnapdListOneRequest (name, d->client);
 }
 
+QSnapdGetAppsRequest::~QSnapdGetAppsRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdGetAppsRequest *QSnapdClient::getApps ()
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetAppsRequest (0, d->client);
+}
+
+QSnapdGetAppsRequest *QSnapdClient::getApps (GetAppsFlags flags)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetAppsRequest (flags, d->client);
+}
+
 QSnapdGetIconRequest::~QSnapdGetIconRequest ()
 {
     delete d_ptr;
@@ -388,6 +456,17 @@ QSnapdDisableRequest *QSnapdClient::disable (const QString& name)
     return new QSnapdDisableRequest (name, d->client);
 }
 
+QSnapdSwitchChannelRequest::~QSnapdSwitchChannelRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdSwitchChannelRequest *QSnapdClient::switchChannel (const QString& name, const QString& channel)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdSwitchChannelRequest (name, channel, d->client);
+}
+
 QSnapdCheckBuyRequest::~QSnapdCheckBuyRequest ()
 {
     delete d_ptr;
@@ -410,6 +489,45 @@ QSnapdBuyRequest *QSnapdClient::buy (const QString& id, double amount, const QSt
     return new QSnapdBuyRequest (id, amount, currency, d->client);
 }
 
+QSnapdCreateUserRequest::~QSnapdCreateUserRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdCreateUserRequest *QSnapdClient::createUser (const QString& email)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdCreateUserRequest (email, 0, d->client);
+}
+
+QSnapdCreateUserRequest *QSnapdClient::createUser (const QString& email, CreateUserFlags flags)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdCreateUserRequest (email, flags, d->client);
+}
+
+QSnapdCreateUsersRequest::~QSnapdCreateUsersRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdCreateUsersRequest *QSnapdClient::createUsers ()
+{
+    Q_D(QSnapdClient);
+    return new QSnapdCreateUsersRequest (d->client);
+}
+
+QSnapdGetUsersRequest::~QSnapdGetUsersRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdGetUsersRequest *QSnapdClient::getUsers ()
+{
+    Q_D(QSnapdClient);
+    return new QSnapdGetUsersRequest (d->client);
+}
+
 QSnapdGetSectionsRequest::~QSnapdGetSectionsRequest ()
 {
     delete d_ptr;
@@ -432,6 +550,45 @@ QSnapdGetAliasesRequest *QSnapdClient::getAliases ()
     return new QSnapdGetAliasesRequest (d->client);
 }
 
+QSnapdAliasRequest::~QSnapdAliasRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdAliasRequest *QSnapdClient::alias (const QString &snap, const QString &app, const QString &alias)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdAliasRequest (snap, app, alias, d->client);
+}
+
+QSnapdUnaliasRequest::~QSnapdUnaliasRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdUnaliasRequest *QSnapdClient::unalias (const QString &snap, const QString &alias)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdUnaliasRequest (snap, alias, d->client);
+}
+
+QSnapdUnaliasRequest *QSnapdClient::unalias (const QString &alias)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdUnaliasRequest (NULL, alias, d->client);
+}
+
+QSnapdPreferRequest::~QSnapdPreferRequest ()
+{
+    delete d_ptr;
+}
+
+QSnapdPreferRequest *QSnapdClient::prefer (const QString &snap)
+{
+    Q_D(QSnapdClient);
+    return new QSnapdPreferRequest (snap, d->client);
+}
+
 QSnapdEnableAliasesRequest::~QSnapdEnableAliasesRequest ()
 {
     delete d_ptr;
@@ -495,9 +652,9 @@ void QSnapdConnectRequest::runAsync ()
     handleResult (NULL, NULL);
 }
 
-QSnapdLoginRequest::QSnapdLoginRequest (void *snapd_client, const QString& username, const QString& password, const QString& otp, QObject *parent) :
+QSnapdLoginRequest::QSnapdLoginRequest (void *snapd_client, const QString& email, const QString& password, const QString& otp, QObject *parent) :
     QSnapdRequest (snapd_client, parent),
-    d_ptr (new QSnapdLoginRequestPrivate(username, password, otp))
+    d_ptr (new QSnapdLoginRequestPrivate(email, password, otp))
 {
 }
 
@@ -506,21 +663,22 @@ void QSnapdLoginRequest::runSync ()
     Q_D(QSnapdLoginRequest);
     g_autoptr(GError) error = NULL;
     if (getClient () != NULL)
-        d->auth_data = snapd_client_login_sync (SNAPD_CLIENT (getClient ()), d->username.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
+        d->user_information = snapd_client_login2_sync (SNAPD_CLIENT (getClient ()), d->email.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
     else
-        d->auth_data = snapd_login_sync (d->username.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
+        d->auth_data = snapd_login_sync (d->email.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
     finish (error);
 }
 
 void QSnapdLoginRequest::handleResult (void *object, void *result)
 {
-    g_autoptr(SnapdAuthData) auth_data = NULL;
+    Q_D(QSnapdLoginRequest);
     g_autoptr(GError) error = NULL;
 
-    auth_data = snapd_client_login_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+    if (getClient () != NULL)
+        d->user_information = snapd_client_login2_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+    else
+        d->auth_data = snapd_login_finish (G_ASYNC_RESULT (result), &error);
 
-    Q_D(QSnapdLoginRequest);
-    d->auth_data = (SnapdAuthData*) g_steal_pointer (&auth_data);
     finish (error);
 }
 
@@ -534,15 +692,172 @@ void QSnapdLoginRequest::runAsync ()
 {
     Q_D(QSnapdLoginRequest);
     if (getClient () != NULL)
-        snapd_client_login_async (SNAPD_CLIENT (getClient ()), d->username.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), login_ready_cb, (gpointer) this);
+        snapd_client_login2_async (SNAPD_CLIENT (getClient ()), d->email.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), login_ready_cb, (gpointer) this);
     else
-        snapd_login_async (d->username.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), login_ready_cb, (gpointer) this);
+        snapd_login_async (d->email.toStdString ().c_str (), d->password.toStdString ().c_str (), d->otp.isNull () ? NULL : d->otp.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), login_ready_cb, (gpointer) this);
+}
+
+QSnapdUserInformation *QSnapdLoginRequest::userInformation ()
+{
+    Q_D(QSnapdLoginRequest);
+    return new QSnapdUserInformation (d->user_information);
 }
 
 QSnapdAuthData *QSnapdLoginRequest::authData ()
 {
     Q_D(QSnapdLoginRequest);
-    return new QSnapdAuthData (d->auth_data);
+    if (d->auth_data != NULL)
+        return new QSnapdAuthData (d->auth_data);
+    else
+        return new QSnapdAuthData (snapd_user_information_get_auth_data (d->user_information));
+}
+
+static SnapdChangeFilter convertChangeFilter (int filter)
+{
+    switch (filter)
+    {
+    default:
+    case QSnapdClient::ChangeFilter::FilterAll:
+        return SNAPD_CHANGE_FILTER_ALL;
+    case QSnapdClient::ChangeFilter::FilterInProgress:
+        return SNAPD_CHANGE_FILTER_IN_PROGRESS;
+    case QSnapdClient::ChangeFilter::FilterReady:
+        return SNAPD_CHANGE_FILTER_READY;
+    }
+}
+
+QSnapdGetChangesRequest::QSnapdGetChangesRequest (int filter, const QString& snapName, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdGetChangesRequestPrivate (filter, snapName)) {}
+
+void QSnapdGetChangesRequest::runSync ()
+{
+    Q_D(QSnapdGetChangesRequest);
+    g_autoptr(GError) error = NULL;
+    d->changes = snapd_client_get_changes_sync (SNAPD_CLIENT (getClient ()), convertChangeFilter (d->filter), d->snapName.isNull () ? NULL : d->snapName.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdGetChangesRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(GPtrArray) changes = NULL;
+    g_autoptr(GError) error = NULL;
+
+    changes = snapd_client_get_changes_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    Q_D(QSnapdGetChangesRequest);
+    d->changes = (GPtrArray*) g_steal_pointer (&changes);
+    finish (error);
+}
+
+static void changes_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdGetChangesRequest *request = static_cast<QSnapdGetChangesRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdGetChangesRequest::runAsync ()
+{
+    Q_D(QSnapdGetChangesRequest);
+    snapd_client_get_changes_async (SNAPD_CLIENT (getClient ()), convertChangeFilter (d->filter), d->snapName.isNull () ? NULL : d->snapName.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), changes_ready_cb, (gpointer) this);
+}
+
+int QSnapdGetChangesRequest::changeCount () const
+{
+    Q_D(const QSnapdGetChangesRequest);
+    return d->changes != NULL ? d->changes->len : 0;
+}
+
+QSnapdChange *QSnapdGetChangesRequest::change (int n) const
+{
+    Q_D(const QSnapdGetChangesRequest);
+    if (d->changes == NULL || n < 0 || (guint) n >= d->changes->len)
+        return NULL;
+    return new QSnapdChange (d->changes->pdata[n]);
+}
+
+QSnapdGetChangeRequest::QSnapdGetChangeRequest (const QString& id, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdGetChangeRequestPrivate (id)) {}
+
+void QSnapdGetChangeRequest::runSync ()
+{
+    Q_D(QSnapdGetChangeRequest);
+    g_autoptr(GError) error = NULL;
+    d->change = snapd_client_get_change_sync (SNAPD_CLIENT (getClient ()), d->id.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdGetChangeRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(SnapdChange) change = NULL;
+    g_autoptr(GError) error = NULL;
+
+    change = snapd_client_get_change_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    Q_D(QSnapdGetChangeRequest);
+    d->change = (SnapdChange*) g_steal_pointer (&change);
+    finish (error);
+}
+
+static void change_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdGetChangeRequest *request = static_cast<QSnapdGetChangeRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdGetChangeRequest::runAsync ()
+{
+    Q_D(QSnapdGetChangeRequest);
+    snapd_client_get_change_async (SNAPD_CLIENT (getClient ()), d->id.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), change_ready_cb, (gpointer) this);
+}
+
+QSnapdChange *QSnapdGetChangeRequest::change () const
+{
+    Q_D(const QSnapdGetChangeRequest);
+    return new QSnapdChange (d->change);
+}
+
+QSnapdAbortChangeRequest::QSnapdAbortChangeRequest (const QString& id, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdAbortChangeRequestPrivate (id)) {}
+
+void QSnapdAbortChangeRequest::runSync ()
+{
+    Q_D(QSnapdAbortChangeRequest);
+    g_autoptr(GError) error = NULL;
+    d->change = snapd_client_abort_change_sync (SNAPD_CLIENT (getClient ()), d->id.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdAbortChangeRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(SnapdChange) change = NULL;
+    g_autoptr(GError) error = NULL;
+
+    change = snapd_client_abort_change_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    Q_D(QSnapdAbortChangeRequest);
+    d->change = (SnapdChange*) g_steal_pointer (&change);
+    finish (error);
+}
+
+static void abort_change_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdAbortChangeRequest *request = static_cast<QSnapdAbortChangeRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdAbortChangeRequest::runAsync ()
+{
+    Q_D(QSnapdAbortChangeRequest);
+    snapd_client_abort_change_async (SNAPD_CLIENT (getClient ()), d->id.toStdString ().c_str (), G_CANCELLABLE (getCancellable ()), abort_change_ready_cb, (gpointer) this);
+}
+
+QSnapdChange *QSnapdAbortChangeRequest::change () const
+{
+    Q_D(const QSnapdAbortChangeRequest);
+    return new QSnapdChange (d->change);
 }
 
 QSnapdGetSystemInformationRequest::QSnapdGetSystemInformationRequest (void *snapd_client, QObject *parent) :
@@ -677,6 +992,66 @@ QSnapdSnap *QSnapdListOneRequest::snap () const
     return new QSnapdSnap (d->snap);
 }
 
+static SnapdGetAppsFlags convertGetAppsFlags (int flags)
+{
+    int result = SNAPD_GET_APPS_FLAGS_NONE;
+
+    if ((flags & QSnapdClient::GetAppsFlag::SelectServices) != 0)
+        result |= SNAPD_GET_APPS_FLAGS_SELECT_SERVICES;
+
+    return (SnapdGetAppsFlags) result;
+}
+
+QSnapdGetAppsRequest::QSnapdGetAppsRequest (int flags, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdGetAppsRequestPrivate (flags)) {}
+
+void QSnapdGetAppsRequest::runSync ()
+{
+    Q_D(QSnapdGetAppsRequest);
+    g_autoptr(GError) error = NULL;
+    d->apps = snapd_client_get_apps_sync (SNAPD_CLIENT (getClient ()), convertGetAppsFlags (d->flags), G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdGetAppsRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(GPtrArray) apps = NULL;
+    g_autoptr(GError) error = NULL;
+
+    apps = snapd_client_get_apps_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    Q_D(QSnapdGetAppsRequest);
+    d->apps = (GPtrArray*) g_steal_pointer (&apps);
+    finish (error);
+}
+
+static void get_apps_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdGetAppsRequest *request = static_cast<QSnapdGetAppsRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdGetAppsRequest::runAsync ()
+{
+    Q_D(QSnapdGetAppsRequest);
+    snapd_client_get_apps_async (SNAPD_CLIENT (getClient ()), convertGetAppsFlags (d->flags), G_CANCELLABLE (getCancellable ()), get_apps_ready_cb, (gpointer) this);
+}
+
+int QSnapdGetAppsRequest::appCount () const
+{
+    Q_D(const QSnapdGetAppsRequest);
+    return d->apps != NULL ? d->apps->len : 0;
+}
+
+QSnapdApp *QSnapdGetAppsRequest::app (int n) const
+{
+    Q_D(const QSnapdGetAppsRequest);
+    if (d->apps == NULL || n < 0 || (guint) n >= d->apps->len)
+        return NULL;
+    return new QSnapdApp (d->apps->pdata[n]);
+}
+
 QSnapdGetIconRequest::QSnapdGetIconRequest (const QString& name, void *snapd_client, QObject *parent) :
     QSnapdRequest (snapd_client, parent),
     d_ptr (new QSnapdGetIconRequestPrivate (name)) {}
@@ -1417,6 +1792,47 @@ void QSnapdDisableRequest::runAsync ()
                                 G_CANCELLABLE (getCancellable ()), disable_ready_cb, (gpointer) this);
 }
 
+QSnapdSwitchChannelRequest::QSnapdSwitchChannelRequest (const QString& name, const QString& channel, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdSwitchChannelRequestPrivate (name, channel)) {}
+
+void QSnapdSwitchChannelRequest::runSync ()
+{
+    Q_D(QSnapdSwitchChannelRequest);
+    g_autoptr(GError) error = NULL;
+    snapd_client_switch_sync (SNAPD_CLIENT (getClient ()),
+                              d->name.toStdString ().c_str (),
+                              d->channel.toStdString ().c_str (),
+                              progress_cb, this,
+                              G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdSwitchChannelRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(GError) error = NULL;
+
+    snapd_client_switch_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void switch_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdSwitchChannelRequest *request = static_cast<QSnapdSwitchChannelRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdSwitchChannelRequest::runAsync ()
+{
+    Q_D(QSnapdSwitchChannelRequest);
+    snapd_client_switch_async (SNAPD_CLIENT (getClient ()),
+                               d->name.toStdString ().c_str (),
+                               d->channel.toStdString ().c_str (),
+                               progress_cb, this,
+                               G_CANCELLABLE (getCancellable ()), switch_ready_cb, (gpointer) this);
+}
+
 QSnapdCheckBuyRequest::QSnapdCheckBuyRequest (void *snapd_client, QObject *parent):
     QSnapdRequest (snapd_client, parent),
     d_ptr (new QSnapdCheckBuyRequestPrivate ()) {}
@@ -1497,6 +1913,160 @@ void QSnapdBuyRequest::runAsync ()
                             G_CANCELLABLE (getCancellable ()), buy_ready_cb, (gpointer) this);
 }
 
+static SnapdCreateUserFlags convertCreateUserFlags (int flags)
+{
+    int result = SNAPD_CREATE_USER_FLAGS_NONE;
+
+    if ((flags & QSnapdClient::CreateUserFlag::Sudo) != 0)
+        result |= SNAPD_CREATE_USER_FLAGS_SUDO;
+    if ((flags & QSnapdClient::CreateUserFlag::Known) != 0)
+        result |= SNAPD_CREATE_USER_FLAGS_KNOWN;
+
+    return (SnapdCreateUserFlags) result;
+}
+
+QSnapdCreateUserRequest::QSnapdCreateUserRequest (const QString& email, int flags, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdCreateUserRequestPrivate (email, flags)) {}
+
+void QSnapdCreateUserRequest::runSync ()
+{
+    Q_D(QSnapdCreateUserRequest);
+    g_autoptr(GError) error = NULL;
+    d->info = snapd_client_create_user_sync (SNAPD_CLIENT (getClient ()),
+                                             d->email.toStdString ().c_str (), convertCreateUserFlags (d->flags),
+                                             G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdCreateUserRequest::handleResult (void *object, void *result)
+{
+    Q_D(QSnapdCreateUserRequest);
+    g_autoptr(GError) error = NULL;
+
+    d->info = snapd_client_create_user_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void create_user_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdCreateUserRequest *request = static_cast<QSnapdCreateUserRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdCreateUserRequest::runAsync ()
+{
+    Q_D(QSnapdCreateUserRequest);
+    snapd_client_create_user_async (SNAPD_CLIENT (getClient ()),
+                                    d->email.toStdString ().c_str (), convertCreateUserFlags (d->flags),
+                                    G_CANCELLABLE (getCancellable ()), create_user_ready_cb, (gpointer) this);
+}
+
+QSnapdUserInformation *QSnapdCreateUserRequest::userInformation () const
+{
+    Q_D(const QSnapdCreateUserRequest);
+    return new QSnapdUserInformation (d->info);
+}
+
+QSnapdCreateUsersRequest::QSnapdCreateUsersRequest (void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdCreateUsersRequestPrivate ()) {}
+
+void QSnapdCreateUsersRequest::runSync ()
+{
+    Q_D(QSnapdCreateUsersRequest);
+    g_autoptr(GError) error = NULL;
+    d->info = snapd_client_create_users_sync (SNAPD_CLIENT (getClient ()),
+                                              G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdCreateUsersRequest::handleResult (void *object, void *result)
+{
+    Q_D(QSnapdCreateUsersRequest);
+    g_autoptr(GError) error = NULL;
+
+    d->info = snapd_client_create_users_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void create_users_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdCreateUsersRequest *request = static_cast<QSnapdCreateUsersRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdCreateUsersRequest::runAsync ()
+{
+    snapd_client_create_users_async (SNAPD_CLIENT (getClient ()),
+                                     G_CANCELLABLE (getCancellable ()), create_users_ready_cb, (gpointer) this);
+}
+
+int QSnapdCreateUsersRequest::userInformationCount () const
+{
+    Q_D(const QSnapdCreateUsersRequest);
+    return d->info != NULL ? d->info->len : 0;
+}
+
+QSnapdUserInformation *QSnapdCreateUsersRequest::userInformation (int n) const
+{
+    Q_D(const QSnapdCreateUsersRequest);
+    if (d->info == NULL || n < 0 || (guint) n >= d->info->len)
+        return NULL;
+    return new QSnapdUserInformation (d->info->pdata[n]);
+}
+
+QSnapdGetUsersRequest::QSnapdGetUsersRequest (void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdGetUsersRequestPrivate ()) {}
+
+void QSnapdGetUsersRequest::runSync ()
+{
+    Q_D(QSnapdGetUsersRequest);
+    g_autoptr(GError) error = NULL;
+    d->info = snapd_client_get_users_sync (SNAPD_CLIENT (getClient ()),
+                                           G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdGetUsersRequest::handleResult (void *object, void *result)
+{
+    Q_D(QSnapdGetUsersRequest);
+    g_autoptr(GError) error = NULL;
+
+    d->info = snapd_client_get_users_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void get_users_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdGetUsersRequest *request = static_cast<QSnapdGetUsersRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdGetUsersRequest::runAsync ()
+{
+    snapd_client_get_users_async (SNAPD_CLIENT (getClient ()),
+                                  G_CANCELLABLE (getCancellable ()), get_users_ready_cb, (gpointer) this);
+}
+
+int QSnapdGetUsersRequest::userInformationCount () const
+{
+    Q_D(const QSnapdGetUsersRequest);
+    return d->info != NULL ? d->info->len : 0;
+}
+
+QSnapdUserInformation *QSnapdGetUsersRequest::userInformation (int n) const
+{
+    Q_D(const QSnapdGetUsersRequest);
+    if (d->info == NULL || n < 0 || (guint) n >= d->info->len)
+        return NULL;
+    return new QSnapdUserInformation (d->info->pdata[n]);
+}
+
 QSnapdGetSectionsRequest::QSnapdGetSectionsRequest (void *snapd_client, QObject *parent) :
     QSnapdRequest (snapd_client, parent),
     d_ptr (new QSnapdGetSectionsRequestPrivate ()) {}
@@ -1594,6 +2164,132 @@ QSnapdAlias *QSnapdGetAliasesRequest::alias (int n) const
     return new QSnapdAlias (d->aliases->pdata[n]);
 }
 
+QSnapdAliasRequest::QSnapdAliasRequest (const QString& snap, const QString& app, const QString& alias, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdAliasRequestPrivate (snap, app, alias)) {}
+
+void QSnapdAliasRequest::runSync ()
+{
+    Q_D(QSnapdAliasRequest);
+    g_autoptr(GError) error = NULL;
+
+    snapd_client_alias_sync (SNAPD_CLIENT (getClient ()),
+                             d->snap.toStdString ().c_str (),
+                             d->app.toStdString ().c_str (),
+                             d->alias.toStdString ().c_str (),
+                             progress_cb, this,
+                             G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdAliasRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(GError) error = NULL;
+    snapd_client_alias_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void alias_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdAliasRequest *request = static_cast<QSnapdAliasRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdAliasRequest::runAsync ()
+{
+    Q_D(QSnapdAliasRequest);
+
+    snapd_client_alias_async (SNAPD_CLIENT (getClient ()),
+                              d->snap.toStdString ().c_str (),
+                              d->app.toStdString ().c_str (),
+                              d->alias.toStdString ().c_str (),
+                              progress_cb, this,
+                              G_CANCELLABLE (getCancellable ()), alias_ready_cb, (gpointer) this);
+}
+
+QSnapdUnaliasRequest::QSnapdUnaliasRequest (const QString& snap, const QString& alias, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdUnaliasRequestPrivate (snap, alias)) {}
+
+void QSnapdUnaliasRequest::runSync ()
+{
+    Q_D(QSnapdUnaliasRequest);
+    g_autoptr(GError) error = NULL;
+
+    snapd_client_unalias_sync (SNAPD_CLIENT (getClient ()),
+                               d->snap.isNull () ? NULL : d->snap.toStdString ().c_str (),
+                               d->alias.isNull () ? NULL : d->alias.toStdString ().c_str (),
+                               progress_cb, this,
+                               G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdUnaliasRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(GError) error = NULL;
+    snapd_client_unalias_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void unalias_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdUnaliasRequest *request = static_cast<QSnapdUnaliasRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdUnaliasRequest::runAsync ()
+{
+    Q_D(QSnapdUnaliasRequest);
+
+    snapd_client_unalias_async (SNAPD_CLIENT (getClient ()),
+                                d->snap.isNull () ? NULL : d->snap.toStdString ().c_str (),
+                                d->alias.isNull () ? NULL : d->alias.toStdString ().c_str (),
+                                progress_cb, this,
+                                G_CANCELLABLE (getCancellable ()), unalias_ready_cb, (gpointer) this);
+}
+
+QSnapdPreferRequest::QSnapdPreferRequest (const QString& snap, void *snapd_client, QObject *parent) :
+    QSnapdRequest (snapd_client, parent),
+    d_ptr (new QSnapdPreferRequestPrivate (snap)) {}
+
+void QSnapdPreferRequest::runSync ()
+{
+    Q_D(QSnapdPreferRequest);
+    g_autoptr(GError) error = NULL;
+
+    snapd_client_prefer_sync (SNAPD_CLIENT (getClient ()),
+                              d->snap.toStdString ().c_str (),
+                              progress_cb, this,
+                              G_CANCELLABLE (getCancellable ()), &error);
+    finish (error);
+}
+
+void QSnapdPreferRequest::handleResult (void *object, void *result)
+{
+    g_autoptr(GError) error = NULL;
+    snapd_client_prefer_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+
+    finish (error);
+}
+
+static void prefer_ready_cb (GObject *object, GAsyncResult *result, gpointer data)
+{
+    QSnapdPreferRequest *request = static_cast<QSnapdPreferRequest*>(data);
+    request->handleResult (object, result);
+}
+
+void QSnapdPreferRequest::runAsync ()
+{
+    Q_D(QSnapdPreferRequest);
+
+    snapd_client_prefer_async (SNAPD_CLIENT (getClient ()),
+                               d->snap.toStdString ().c_str (),
+                               progress_cb, this,
+                               G_CANCELLABLE (getCancellable ()), prefer_ready_cb, (gpointer) this);
+}
+
 QSnapdEnableAliasesRequest::QSnapdEnableAliasesRequest (const QString& name, const QStringList& aliases, void *snapd_client, QObject *parent) :
     QSnapdRequest (snapd_client, parent),
     d_ptr (new QSnapdEnableAliasesRequestPrivate (name, aliases)) {}
@@ -1605,17 +2301,21 @@ void QSnapdEnableAliasesRequest::runSync ()
     g_autoptr(GError) error = NULL;
 
     aliases = string_list_to_strv (d->aliases);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_enable_aliases_sync (SNAPD_CLIENT (getClient ()),
                                       d->snap.toStdString ().c_str (), aliases,
                                       progress_cb, this,
                                       G_CANCELLABLE (getCancellable ()), &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
     finish (error);
 }
 
 void QSnapdEnableAliasesRequest::handleResult (void *object, void *result)
 {
     g_autoptr(GError) error = NULL;
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_enable_aliases_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     finish (error);
 }
@@ -1632,10 +2332,12 @@ void QSnapdEnableAliasesRequest::runAsync ()
     g_auto(GStrv) aliases = NULL;
 
     aliases = string_list_to_strv (d->aliases);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_disable_aliases_async (SNAPD_CLIENT (getClient ()),
                                        d->snap.toStdString ().c_str (), aliases,
                                        progress_cb, this,
                                        G_CANCELLABLE (getCancellable ()), enable_aliases_ready_cb, (gpointer) this);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 QSnapdDisableAliasesRequest::QSnapdDisableAliasesRequest (const QString& name, const QStringList& aliases, void *snapd_client, QObject *parent) :
@@ -1649,17 +2351,21 @@ void QSnapdDisableAliasesRequest::runSync ()
     g_autoptr(GError) error = NULL;
 
     aliases = string_list_to_strv (d->aliases);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_disable_aliases_sync (SNAPD_CLIENT (getClient ()),
                                        d->snap.toStdString ().c_str (), aliases,
                                        progress_cb, this,
                                        G_CANCELLABLE (getCancellable ()), &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
     finish (error);
 }
 
 void QSnapdDisableAliasesRequest::handleResult (void *object, void *result)
 {
     g_autoptr(GError) error = NULL;
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_disable_aliases_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     finish (error);
 }
@@ -1676,10 +2382,12 @@ void QSnapdDisableAliasesRequest::runAsync ()
     g_auto(GStrv) aliases = NULL;
 
     aliases = string_list_to_strv (d->aliases);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_disable_aliases_async (SNAPD_CLIENT (getClient ()),
                                        d->snap.toStdString ().c_str (), aliases,
                                        progress_cb, this,
                                        G_CANCELLABLE (getCancellable ()), disable_aliases_ready_cb, (gpointer) this);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 QSnapdResetAliasesRequest::QSnapdResetAliasesRequest (const QString& name, const QStringList& aliases, void *snapd_client, QObject *parent) :
@@ -1693,17 +2401,21 @@ void QSnapdResetAliasesRequest::runSync ()
     g_autoptr(GError) error = NULL;
 
     aliases = string_list_to_strv (d->aliases);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_reset_aliases_sync (SNAPD_CLIENT (getClient ()),
                                      d->snap.toStdString ().c_str (), aliases,
                                      progress_cb, this,
                                      G_CANCELLABLE (getCancellable ()), &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
     finish (error);
 }
 
 void QSnapdResetAliasesRequest::handleResult (void *object, void *result)
 {
     g_autoptr(GError) error = NULL;
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_reset_aliases_finish (SNAPD_CLIENT (object), G_ASYNC_RESULT (result), &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     finish (error);
 }
@@ -1720,10 +2432,12 @@ void QSnapdResetAliasesRequest::runAsync ()
     g_auto(GStrv) aliases = NULL;
 
     aliases = string_list_to_strv (d->aliases);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     snapd_client_reset_aliases_async (SNAPD_CLIENT (getClient ()),
                                       d->snap.toStdString ().c_str (), aliases,
                                       progress_cb, this,
                                       G_CANCELLABLE (getCancellable ()), reset_aliases_ready_cb, (gpointer) this);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 QSnapdRunSnapCtlRequest::QSnapdRunSnapCtlRequest (const QString& contextId, const QStringList& args, void *snapd_client, QObject *parent) :
