@@ -25,124 +25,18 @@ struct _MockSnapdClass
     GObjectClass parent_class;
 };
 
-typedef struct
-{
-    gchar *username;
-    gchar *password;
-    gchar *otp;
-    gchar *macaroon;
-    gchar **discharges;
-    gboolean terms_accepted;
-    gboolean has_payment_methods;
-    GList *private_snaps;
-} MockAccount;
-
-typedef struct
-{
-    GList *apps;
-    gchar *channel;
-    gchar *confinement;
-    gchar *contact;
-    gchar *description;
-    gchar *developer;
-    gboolean devmode;
-    int download_size;
-    gchar *icon;
-    gchar *icon_mime_type;
-    GBytes *icon_data;
-    gchar *id;
-    gchar *install_date;
-    int installed_size;
-    gboolean jailmode;
-    gchar *license;
-    gchar *name;
-    GList *prices;
-    gboolean is_private;
-    gchar *revision;
-    GList *screenshots;
-    gchar *status;
-    gchar *summary;
-    gchar *title;
-    gchar *tracking_channel;
-    GList *tracks;
-    gboolean trymode;
-    gchar *type;
-    gchar *version;
-    GList *store_sections;
-    GList *plugs;
-    GList *slots_;
-    gboolean disabled;
-    gboolean dangerous;
-    gchar *snap_data;
-    gchar *snap_path;
-    gchar *error;
-    gboolean restart_required;
-} MockSnap;
-
-typedef struct
-{
-    gchar *name;
-    GList *aliases;
-    gchar *daemon;
-    gchar *desktop_file;
-} MockApp;
-
-typedef struct
-{
-    gchar *name;
-    gchar *status;
-} MockAlias;
-
-typedef struct
-{
-    gchar *name;
-    GList *channels;
-} MockTrack;
-
-typedef struct
-{
-    gchar *risk;
-    gchar *branch;
-    gchar *confinement;
-    gchar *epoch;
-    gchar *revision;
-    int size;
-    gchar *version;
-} MockChannel;
-
-typedef struct
-{
-    gdouble amount;
-    gchar *currency;
-} MockPrice;
-
-typedef struct
-{
-    gchar *url;
-    int width;
-    int height;
-} MockScreenshot;
-
+typedef struct _MockAccount MockAccount;
+typedef struct _MockAlias MockAlias;
+typedef struct _MockApp MockApp;
+typedef struct _MockChange MockChange;
+typedef struct _MockChannel MockChannel;
+typedef struct _MockPlug MockPlug;
+typedef struct _MockPrice MockPrice;
+typedef struct _MockScreenshot MockScreenshot;
 typedef struct _MockSlot MockSlot;
-
-typedef struct
-{
-    MockSnap *snap;
-    gchar *name;
-    gchar *interface;
-    // FIXME: Attributes
-    gchar *label;
-    MockSlot *connection;
-} MockPlug;
-
-struct _MockSlot
-{
-    MockSnap *snap;
-    gchar *name;
-    gchar *interface;
-    // FIXME: Attributes
-    gchar *label;
-};
+typedef struct _MockSnap MockSnap;
+typedef struct _MockTask MockTask;
+typedef struct _MockTrack MockTrack;
 
 MockSnapd      *mock_snapd_new                    (void);
 
@@ -181,10 +75,63 @@ void            mock_snapd_set_ready_time         (MockSnapd     *snapd,
                                                    const gchar   *ready_time);
 
 MockAccount    *mock_snapd_add_account            (MockSnapd     *snapd,
+                                                   const gchar   *email,
                                                    const gchar   *username,
-                                                   const gchar   *password,
+                                                   const gchar   *password);
+
+void            mock_account_set_terms_accepted   (MockAccount   *account,
+                                                   gboolean       terms_accepted);
+
+void            mock_account_set_has_payment_methods (MockAccount   *account,
+                                                      gboolean       has_payment_methods);
+
+const gchar    *mock_account_get_macaroon         (MockAccount   *account);
+
+gchar         **mock_account_get_discharges       (MockAccount   *account);
+
+gboolean        mock_account_get_sudoer           (MockAccount   *account);
+
+gboolean        mock_account_get_known            (MockAccount   *account);
+
+void            mock_account_set_otp              (MockAccount   *account,
                                                    const gchar   *otp);
 
+void            mock_account_set_ssh_keys         (MockAccount   *account,
+                                                   gchar        **ssh_keys);
+
+MockAccount    *mock_snapd_find_account_by_username (MockSnapd     *snapd,
+                                                     const gchar   *username);
+
+MockAccount    *mock_snapd_find_account_by_email  (MockSnapd     *snapd,
+                                                   const gchar   *email);
+
+MockChange     *mock_snapd_add_change             (MockSnapd     *snapd);
+
+MockTask       *mock_change_add_task              (MockChange    *change,
+                                                   const gchar   *kind);
+
+void            mock_task_set_snap_name           (MockTask      *task,
+                                                   const gchar   *snap_name);
+
+void            mock_task_set_status              (MockTask      *task,
+                                                   const gchar   *status);
+
+void            mock_task_set_progress            (MockTask      *task,
+                                                   int            done,
+                                                   int            total);
+
+void            mock_task_set_spawn_time          (MockTask      *task,
+                                                   const gchar   *spawn_time);
+
+void            mock_task_set_ready_time          (MockTask      *task,
+                                                   const gchar   *ready_time);
+
+void            mock_change_set_spawn_time        (MockChange    *change,
+                                                   const gchar   *spawn_time);
+
+void            mock_change_set_ready_time        (MockChange    *change,
+                                                   const gchar   *ready_time);
+
 MockSnap       *mock_account_add_private_snap     (MockAccount   *account,
                                                    const gchar   *name);
 
@@ -203,11 +150,11 @@ MockSnap       *mock_snapd_add_store_snap         (MockSnapd     *snapd,
 MockApp        *mock_snap_add_app                 (MockSnap      *snap,
                                                    const gchar   *name);
 
-MockAlias      *mock_app_add_alias                (MockApp       *app,
-                                                   const gchar   *alias);
+void            mock_app_set_active               (MockApp       *app,
+                                                   gboolean       active);
 
-void            mock_alias_set_status             (MockAlias     *alias,
-                                                   const gchar   *status);
+void            mock_app_set_enabled              (MockApp       *app,
+                                                   gboolean       enabled);
 
 void            mock_app_set_daemon               (MockApp       *app,
                                                    const gchar   *daemon);
@@ -215,9 +162,24 @@ void            mock_app_set_daemon               (MockApp       *app,
 void            mock_app_set_desktop_file         (MockApp       *app,
                                                    const gchar   *desktop_file);
 
+void            mock_app_add_auto_alias           (MockApp       *app,
+                                                   const gchar   *alias);
+
+void            mock_app_add_manual_alias         (MockApp       *app,
+                                                   const gchar   *alias,
+                                                   gboolean       enabled);
+
+MockAlias      *mock_app_find_alias               (MockApp       *app,
+                                                   const gchar   *name);
+
+void            mock_snap_set_broken              (MockSnap      *snap,
+                                                   const gchar   *broken);
+
 void            mock_snap_set_channel             (MockSnap      *snap,
                                                    const gchar   *channel);
 
+const gchar    *mock_snap_get_channel             (MockSnap      *snap);
+
 MockTrack      *mock_snap_add_track               (MockSnap      *snap,
                                                    const gchar   *name);
 
@@ -234,21 +196,43 @@ void            mock_channel_set_epoch            (MockChannel  *channel,
 void            mock_channel_set_revision         (MockChannel  *channel,
                                                    const gchar   *revision);
 
+void            mock_channel_set_size             (MockChannel  *channel,
+                                                   int           size);
+
 void            mock_channel_set_version          (MockChannel  *channel,
                                                    const gchar   *version);
 
 void            mock_snap_set_confinement         (MockSnap      *snap,
                                                    const gchar   *confinement);
 
+const gchar    *mock_snap_get_confinement         (MockSnap      *snap);
+
 void            mock_snap_set_contact             (MockSnap      *snap,
                                                    const gchar   *contact);
 
+gboolean        mock_snap_get_dangerous           (MockSnap      *snap);
+
+const gchar    *mock_snap_get_data                (MockSnap      *snap);
+
 void            mock_snap_set_description         (MockSnap      *snap,
                                                    const gchar   *description);
 
 void            mock_snap_set_developer           (MockSnap      *snap,
                                                    const gchar   *developer);
 
+void            mock_snap_set_devmode             (MockSnap      *snap,
+                                                   gboolean       devmode);
+
+gboolean        mock_snap_get_devmode             (MockSnap      *snap);
+
+void            mock_snap_set_disabled            (MockSnap      *snap,
+                                                   gboolean       disabled);
+
+gboolean        mock_snap_get_disabled            (MockSnap      *snap);
+
+void            mock_snap_set_download_size       (MockSnap      *snap,
+                                                   int           download_size);
+
 void            mock_snap_set_error               (MockSnap      *snap,
                                                    const gchar   *error);
 
@@ -265,16 +249,33 @@ void            mock_snap_set_id                  (MockSnap      *snap,
 void            mock_snap_set_install_date        (MockSnap      *snap,
                                                    const gchar   *install_date);
 
+void            mock_snap_set_installed_size      (MockSnap      *snap,
+                                                   int           installed_size);
+
+void            mock_snap_set_jailmode            (MockSnap      *snap,
+                                                   gboolean       jailmode);
+
+gboolean        mock_snap_get_jailmode            (MockSnap      *snap);
+
 void            mock_snap_set_license             (MockSnap      *snap,
                                                    const gchar   *license);
 
+const gchar    *mock_snap_get_path                (MockSnap      *snap);
+
+gboolean        mock_snap_get_preferred           (MockSnap      *snap);
+
 MockPrice      *mock_snap_add_price               (MockSnap      *snap,
                                                    gdouble        amount,
                                                    const gchar   *currency);
 
+void            mock_snap_set_restart_required    (MockSnap      *snap,
+                                                   gboolean       restart_required);
+
 void            mock_snap_set_revision            (MockSnap      *snap,
                                                    const gchar   *revision);
 
+const gchar    *mock_snap_get_revision            (MockSnap      *snap);
+
 MockScreenshot *mock_snap_add_screenshot          (MockSnap      *snap,
                                                    const gchar   *url,
                                                    int            width,
@@ -292,6 +293,11 @@ void            mock_snap_set_title               (MockSnap      *snap,
 void            mock_snap_set_tracking_channel    (MockSnap      *snap,
                                                    const gchar   *channel);
 
+const gchar    *mock_snap_get_tracking_channel    (MockSnap      *snap);
+
+void            mock_snap_set_trymode             (MockSnap      *snap,
+                                                   gboolean       trymode);
+
 void            mock_snap_set_type                (MockSnap      *snap,
                                                    const gchar   *type);
 
@@ -307,6 +313,11 @@ MockPlug       *mock_snap_add_plug                (MockSnap      *snap,
 MockSlot       *mock_snap_add_slot                (MockSnap      *snap,
                                                    const gchar   *name);
 
+void            mock_plug_set_connection          (MockPlug      *plug,
+                                                   MockSlot      *slot);
+
+MockSlot       *mock_plug_get_connection          (MockPlug      *plug);
+
 void            mock_snapd_add_assertion          (MockSnapd     *snapd,
                                                    const gchar   *assertion);
 
