@@ -1011,6 +1011,206 @@ int InstallMultipleHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a
     }
     return _id;
 }
+struct qt_meta_stringdata_InstallMultipleCancelFirstHandler_t {
+    QByteArrayData data[5];
+    char stringdata0[83];
+};
+#define QT_MOC_LITERAL(idx, ofs, len) \
+    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
+    qptrdiff(offsetof(qt_meta_stringdata_InstallMultipleCancelFirstHandler_t, stringdata0) + ofs \
+        - idx * sizeof(QByteArrayData)) \
+    )
+static const qt_meta_stringdata_InstallMultipleCancelFirstHandler_t qt_meta_stringdata_InstallMultipleCancelFirstHandler = {
+    {
+QT_MOC_LITERAL(0, 0, 33), // "InstallMultipleCancelFirstHan..."
+QT_MOC_LITERAL(1, 34, 15), // "onCompleteSnap1"
+QT_MOC_LITERAL(2, 50, 0), // ""
+QT_MOC_LITERAL(3, 51, 15), // "onCompleteSnap2"
+QT_MOC_LITERAL(4, 67, 15) // "onCompleteSnap3"
+
+    },
+    "InstallMultipleCancelFirstHandler\0"
+    "onCompleteSnap1\0\0onCompleteSnap2\0"
+    "onCompleteSnap3"
+};
+#undef QT_MOC_LITERAL
+
+static const uint qt_meta_data_InstallMultipleCancelFirstHandler[] = {
+
+ // content:
+       7,       // revision
+       0,       // classname
+       0,    0, // classinfo
+       3,   14, // methods
+       0,    0, // properties
+       0,    0, // enums/sets
+       0,    0, // constructors
+       0,       // flags
+       0,       // signalCount
+
+ // slots: name, argc, parameters, tag, flags
+       1,    0,   29,    2, 0x0a /* Public */,
+       3,    0,   30,    2, 0x0a /* Public */,
+       4,    0,   31,    2, 0x0a /* Public */,
+
+ // slots: parameters
+    QMetaType::Void,
+    QMetaType::Void,
+    QMetaType::Void,
+
+       0        // eod
+};
+
+void InstallMultipleCancelFirstHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
+{
+    if (_c == QMetaObject::InvokeMetaMethod) {
+        InstallMultipleCancelFirstHandler *_t = static_cast<InstallMultipleCancelFirstHandler *>(_o);
+        Q_UNUSED(_t)
+        switch (_id) {
+        case 0: _t->onCompleteSnap1(); break;
+        case 1: _t->onCompleteSnap2(); break;
+        case 2: _t->onCompleteSnap3(); break;
+        default: ;
+        }
+    }
+    Q_UNUSED(_a);
+}
+
+const QMetaObject InstallMultipleCancelFirstHandler::staticMetaObject = {
+    { &QObject::staticMetaObject, qt_meta_stringdata_InstallMultipleCancelFirstHandler.data,
+      qt_meta_data_InstallMultipleCancelFirstHandler,  qt_static_metacall, nullptr, nullptr}
+};
+
+
+const QMetaObject *InstallMultipleCancelFirstHandler::metaObject() const
+{
+    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
+}
+
+void *InstallMultipleCancelFirstHandler::qt_metacast(const char *_clname)
+{
+    if (!_clname) return nullptr;
+    if (!strcmp(_clname, qt_meta_stringdata_InstallMultipleCancelFirstHandler.stringdata0))
+        return static_cast<void*>(const_cast< InstallMultipleCancelFirstHandler*>(this));
+    return QObject::qt_metacast(_clname);
+}
+
+int InstallMultipleCancelFirstHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
+{
+    _id = QObject::qt_metacall(_c, _id, _a);
+    if (_id < 0)
+        return _id;
+    if (_c == QMetaObject::InvokeMetaMethod) {
+        if (_id < 3)
+            qt_static_metacall(this, _c, _id, _a);
+        _id -= 3;
+    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
+        if (_id < 3)
+            *reinterpret_cast<int*>(_a[0]) = -1;
+        _id -= 3;
+    }
+    return _id;
+}
+struct qt_meta_stringdata_InstallMultipleCancelLastHandler_t {
+    QByteArrayData data[5];
+    char stringdata0[82];
+};
+#define QT_MOC_LITERAL(idx, ofs, len) \
+    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
+    qptrdiff(offsetof(qt_meta_stringdata_InstallMultipleCancelLastHandler_t, stringdata0) + ofs \
+        - idx * sizeof(QByteArrayData)) \
+    )
+static const qt_meta_stringdata_InstallMultipleCancelLastHandler_t qt_meta_stringdata_InstallMultipleCancelLastHandler = {
+    {
+QT_MOC_LITERAL(0, 0, 32), // "InstallMultipleCancelLastHandler"
+QT_MOC_LITERAL(1, 33, 15), // "onCompleteSnap1"
+QT_MOC_LITERAL(2, 49, 0), // ""
+QT_MOC_LITERAL(3, 50, 15), // "onCompleteSnap2"
+QT_MOC_LITERAL(4, 66, 15) // "onCompleteSnap3"
+
+    },
+    "InstallMultipleCancelLastHandler\0"
+    "onCompleteSnap1\0\0onCompleteSnap2\0"
+    "onCompleteSnap3"
+};
+#undef QT_MOC_LITERAL
+
+static const uint qt_meta_data_InstallMultipleCancelLastHandler[] = {
+
+ // content:
+       7,       // revision
+       0,       // classname
+       0,    0, // classinfo
+       3,   14, // methods
+       0,    0, // properties
+       0,    0, // enums/sets
+       0,    0, // constructors
+       0,       // flags
+       0,       // signalCount
+
+ // slots: name, argc, parameters, tag, flags
+       1,    0,   29,    2, 0x0a /* Public */,
+       3,    0,   30,    2, 0x0a /* Public */,
+       4,    0,   31,    2, 0x0a /* Public */,
+
+ // slots: parameters
+    QMetaType::Void,
+    QMetaType::Void,
+    QMetaType::Void,
+
+       0        // eod
+};
+
+void InstallMultipleCancelLastHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
+{
+    if (_c == QMetaObject::InvokeMetaMethod) {
+        InstallMultipleCancelLastHandler *_t = static_cast<InstallMultipleCancelLastHandler *>(_o);
+        Q_UNUSED(_t)
+        switch (_id) {
+        case 0: _t->onCompleteSnap1(); break;
+        case 1: _t->onCompleteSnap2(); break;
+        case 2: _t->onCompleteSnap3(); break;
+        default: ;
+        }
+    }
+    Q_UNUSED(_a);
+}
+
+const QMetaObject InstallMultipleCancelLastHandler::staticMetaObject = {
+    { &QObject::staticMetaObject, qt_meta_stringdata_InstallMultipleCancelLastHandler.data,
+      qt_meta_data_InstallMultipleCancelLastHandler,  qt_static_metacall, nullptr, nullptr}
+};
+
+
+const QMetaObject *InstallMultipleCancelLastHandler::metaObject() const
+{
+    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
+}
+
+void *InstallMultipleCancelLastHandler::qt_metacast(const char *_clname)
+{
+    if (!_clname) return nullptr;
+    if (!strcmp(_clname, qt_meta_stringdata_InstallMultipleCancelLastHandler.stringdata0))
+        return static_cast<void*>(const_cast< InstallMultipleCancelLastHandler*>(this));
+    return QObject::qt_metacast(_clname);
+}
+
+int InstallMultipleCancelLastHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
+{
+    _id = QObject::qt_metacall(_c, _id, _a);
+    if (_id < 0)
+        return _id;
+    if (_c == QMetaObject::InvokeMetaMethod) {
+        if (_id < 3)
+            qt_static_metacall(this, _c, _id, _a);
+        _id -= 3;
+    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
+        if (_id < 3)
+            *reinterpret_cast<int*>(_a[0]) = -1;
+        _id -= 3;
+    }
+    return _id;
+}
 struct qt_meta_stringdata_RemoveHandler_t {
     QByteArrayData data[3];
     char stringdata0[26];
